import "../stylesheets/PlantDetails.css";
import "font-awesome/css/font-awesome.min.css";
import { useState } from "react";
import { Plant, PlantSchema } from "../utils/schemas";

//TODO translateEnums fixen

function PlantDetails(props: any) {
  const plant: Plant = PlantSchema.parse(props.plant);

  return (
    <>
      <div style={{ display: "flex", flexDirection: "column" }}>
        <div className="plantDetails">
          <h1>{plant.name}</h1>
          <img src={plant.imageUrl}></img>
          <div className="lateinischerName infoBox">
            <div className="title">Lateinischer Name</div>{" "}
            <p>{plant.latinName}</p>
          </div>
          <div className="Beschreibung infoBox">
            <div className="title">Beschreibung</div> <p>{plant.description}</p>
          </div>
          <div className="infoBox">
            <div className="title">Herkunft</div>
            <p>{plant.origin}</p>
          </div>
          <div className="infoBox">
            <div className="title">Anbautipps</div> <p>{plant.growingTips}</p>
          </div>
          <div className="infoBox">
            <div className="title">Anbauzeitraum</div>
            <p>
              Von Kalenderwoche {plant.plantWeekStart} bis Kalenderwoche{" "}
              {plant.plantWeekEnd}
            </p>
          </div>
          <div className="infoBox">
            <div className="title">Erntezeitraum</div>
            <p>
              Von Kalenderwoche {plant.harvestWeekStart} bis Kalenderwoche{" "}
              {plant.harvestWeekEnd}
            </p>
          </div>
          <div className="infoBox">
            <div className="title">Bodentyp</div>
            {plant.groundType}
          </div>
          <div className="infoBox">
            <div className="title">Nährstoffbedarf</div>
            {plant.nutrientDemand}
          </div>
          <div className="infoBox">
            <div className="title">Wasserbedarf</div>
            <p>{plant.waterDemand}</p>
          </div>
          <div className="infoBox">
            <div className="title">Lichtbedarf</div>
            <p>{plant.lightingDemand}</p>
          </div>
        </div>
        <div
          className="buttons"
          style={{
            display: "flex",
            justifyContent: "space-around",
            margin: "0 0 40px 0",
          }}
        >
          <ButtonAddToGarden plantId={plant.id}></ButtonAddToGarden>
          <ButtonAddToWishlist plantId={plant.id}></ButtonAddToWishlist>
        </div>
      </div>
    </>
  );
}

function ButtonAddToGarden({ plantId }: any) {
  const [status, setStatus] = useState<null | String>(null);
  const token: String = "";

  const handleButtonClick = async () => {
    try {
      const response = await fetch(
        `http://localhost:8080/api/v1/garden/add/${plantId}`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
        }
      );
      if (response.status === 201) {
        console.log("Pflanze wurde erfolgreich dem Garten hinzugefügt");
      } else {
        console.log("Unexpected response status:", response.status);
        setStatus("Pflanze konnte nicht dem Garten hinzugefügt werden");
      }
    } catch (error) {
      console.error("Error", error);
      setStatus("Pflanze konnte nicht dem Garten hinzugefügt werden");
    }
  };

  return (
    <>
      {status && (
        <div
          className={`status-message ${
            status.includes("erfolgreich") ? "success-message" : "error-message"
          }`}
        >
          {status}
        </div>
      )}
      <button onClick={handleButtonClick}>
        <i className="fa fa-leaf"></i>
      </button>
    </>
  );
}

//TODO css für status message
function ButtonAddToWishlist({ plantId }: any) {
  const [status, setStatus] = useState<null | String>(null);
  const token: String = "";

  const handleButtonClick = async () => {
    try {
      const response = await fetch(
        `http://localhost:8080/api/v1/wishlist/add/${plantId}`,
        {
          method: "POST",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
        }
      );
      if (response.status === 201) {
        console.log("Pflanze wurde erfolgreich der Wunschliste hinzugefügt");
      } else {
        setStatus("Pflanze konnte nicht der Wunschliste hinzugefügt werden");
      }
    } catch (error) {
      console.error("Error", error);
      setStatus("Pflanze konnte nicht der Wunschliste hinzugefügt werden");
    }
  };

  return (
    <>
      {status && (
        <div
          className={`status-message ${
            status.includes("erfolgreich") ? "success-message" : "error-message"
          }`}
        >
          {" "}
          {status}
        </div>
      )}
      <button onClick={handleButtonClick}>
        <i className="fa fa-heart"></i>
      </button>
    </>
  );
}

export default PlantDetails;
