import "font-awesome/css/font-awesome.min.css";
import "../stylesheets/Suche.css";
import PlantsOverview from "../components/PlantsOverview";
import { useState, useEffect } from "react";
import FilterPage from "../components/FilterPage";
import { Plant } from "../utils/schemas";

function Suche() {
  const [randomPlants, setRandomPlants] = useState<Plant[] | null>(null);
  const [error, setError] = useState<null | String>(null);
  const randomPlantsCount: number = 30;
  const token: String = "";

  const fetchRandomPlants = async () => {
    try {
      const response = await fetch(
        `http://localhost:8080/api/v1/plants/randomPlants/${randomPlantsCount}`,
        {
          method: "GET",
          headers: {
            "Content-Type": "application/json",
            Authorization: `Bearer ${token}`,
          },
        }
      );
      if (response.status == 200) {
        const data = await response.json();
        setRandomPlants(data);
        console.log("Succesfully fetched data");
      } else {
        console.log("Unexpected response status:", response.status);
        setError("Error fetching data");
      }
    } catch (error) {
      console.error("Error", error);
      setError("Error fetching data");
    }
  };

  useEffect(() => {
    fetchRandomPlants();
  }, []);

  return (
    <>
      <SearchBar />
      <div>
        {error ? <div>{error}</div> : <PlantsOverview plants={randomPlants} />}
      </div>
    </>
  );
}

function SearchBar() {
  const [isComponentVisible, setComponentVisible] = useState(false);

  const loadComponent = () => {
    setComponentVisible(!isComponentVisible);
  };

  return (
    <>
      <div className="searchBar">
        <h2>Suche nach einer Pflanze</h2>
        <p>
          und füge diese dann zu deinem Garten oder zu deiner Wunschliste hinzu
        </p>
        <div className="searchFilter">
          <input type="text" placeholder="Pflanze suchen..." />
          <button>
            <i className="fa fa-search"></i>
          </button>
          <button onClick={loadComponent} className="filter">
            <i className="fa fa-filter" />
          </button>
        </div>
        {isComponentVisible && <FilterPage />}
      </div>
    </>
  );
}

export default Suche;
