package hdm.mi.growbros.models.search;

import hdm.mi.growbros.models.plant.Plant;
import jakarta.persistence.criteria.Predicate;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.annotation.Bean;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Component
public class PlantSpecification {
    private final static Logger log = LogManager.getLogger(PlantSpecification.class);

    public Specification<Plant> withSearchCriteria(SearchRequest searchRequest) {
        return (root, query, criteriaBuilder) -> {
            Collection<Predicate> predicatesAND = new ArrayList<>();
            Collection<Predicate> plantWeekONE = new ArrayList<>();
            Collection<Predicate> plantWeekTWO = new ArrayList<>();


            if (StringUtils.hasText(searchRequest.getSearchTerm())) {
                predicatesAND.add(criteriaBuilder.like(criteriaBuilder.lower(root.get("name")),
                        "%" + searchRequest.getSearchTerm().toLowerCase() + "%"));
            }
            if (searchRequest.getGroundType() != null) {
                predicatesAND.add(criteriaBuilder.equal(root.get("groundType"), searchRequest.getGroundType()));
            }
            if (searchRequest.getNutrientDemand() != null) {
                predicatesAND.add(criteriaBuilder.equal(root.get("nutrientDemand"), searchRequest.getNutrientDemand()));
            }

            if (searchRequest.getWaterDemand() != null) {
                predicatesAND.add(criteriaBuilder.equal(root.get("waterDemand"), searchRequest.getWaterDemand()));
            }
            if (searchRequest.getLightingDemand() != null) {
                predicatesAND.add(criteriaBuilder.equal(root.get("lightingDemand"), searchRequest.getLightingDemand()));
            }

            if (searchRequest.getPlantWeekStart() != null && searchRequest.getPlantWeekEnd() != null) {
                int start = searchRequest.getPlantWeekStart();
                int end = searchRequest.getPlantWeekEnd();
                if(start < end) {
                    plantWeekONE.add(criteriaBuilder.lessThanOrEqualTo(root.get("plantWeekStart"), start));
                    plantWeekONE.add(criteriaBuilder.greaterThanOrEqualTo(root.get("plantWeekEnd"), start));
                    plantWeekTWO.add(criteriaBuilder.lessThanOrEqualTo(root.get("plantWeekStart"), end));
                    plantWeekTWO.add(criteriaBuilder.greaterThanOrEqualTo(root.get("plantWeekEnd"), end));
                } else {
                    //TODO fixen
                    //start: 40
                    //end: 10
                    // -> end: 62
                    plantWeekONE.add(criteriaBuilder.lessThanOrEqualTo(root.get("plantWeekStart"), start));
                    plantWeekONE.add(criteriaBuilder.greaterThanOrEqualTo(root.get("plantWeekEnd"), start));
                    plantWeekTWO.add(criteriaBuilder.lessThanOrEqualTo(root.get("plantWeekStart"), end));
                    plantWeekTWO.add(criteriaBuilder.greaterThanOrEqualTo(root.get("plantWeekEnd"), end));
                }
            }

            if (searchRequest.getGrowthDurationMin() != null && searchRequest.getGrowthDurationMax() != null) {
                predicatesAND.add(criteriaBuilder.greaterThanOrEqualTo(root.get("growthDuration"), searchRequest.getGrowthDurationMin()));
                predicatesAND.add(criteriaBuilder.lessThanOrEqualTo(root.get("growthDuration"), searchRequest.getGrowthDurationMax()));
            }
            return criteriaBuilder.and(
                    criteriaBuilder.or(
                            criteriaBuilder.and(
                                    plantWeekONE.toArray(new Predicate[0])),
                            criteriaBuilder.and(
                                    plantWeekTWO.toArray(new Predicate[0]))
                    ),
                    criteriaBuilder.and(
                            predicatesAND.toArray(new Predicate[0]))
            );
        };
    }
}
