package hdm.mi.growbros.service;

import hdm.mi.growbros.models.WishListEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.WishListRepository;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.temporal.WeekFields;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Random;

@Service

public class GrandmaService {



    private final GardenService gardenService;

    private final WishListRepository wishListRepository;


    public GrandmaService (GardenService gardenService , WishListRepository wishListRepository){

        this.gardenService = gardenService;
        this.wishListRepository = wishListRepository;
    }




    public String growingTippDesTages (){
        List<Plant> usersPlants = getPlantsInUserGarden();
        String tipDesTages;
        Random random = new Random();
        int randomNumber = random.nextInt(usersPlants.size());
        tipDesTages =  "Tip des Tages für deinen Garten"  + usersPlants.get(randomNumber).getGrowingTips();
        return tipDesTages;
    }


    public String notifyAboutPlantsReadyToGrow () {
        List<Plant> plantsReadyToGrow = getPlantsReadyToGrow();
        if (plantsReadyToGrow.isEmpty()) {
            return "";
        }
        StringBuilder notification = new StringBuilder("Diese Pflanzen in deiner Wunschliste wären bereit zum Einpflanzen: ");
        int size = plantsReadyToGrow.size();
        for (int i = 0; i < size; i++) {
            notification.append(plantsReadyToGrow.get(i).getName());
            if (i < size - 2) {
                notification.append(", ");
            } else if (i == size - 2) {
                notification.append(" und ");
            }
        }
        return notification.toString();
    }



    private List<Plant> getPlantsReadyToGrow(){
        LocalDate today = LocalDate.now();
        int weekNumber = today.get(WeekFields.of(Locale.getDefault()).weekOfWeekBasedYear());
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        List<Plant> plantsReadyToGrow = new ArrayList<>();

        if (authentication != null) {
            User user = (User) authentication.getPrincipal();
            List<WishListEntry> wishListEntries = wishListRepository.findByCurrentPlantingWeek(user,weekNumber);
            for (WishListEntry entry : wishListEntries) {
                plantsReadyToGrow.add(entry.getPlant());
            }
        }
        return plantsReadyToGrow;
    }




    private List<Plant> getPlantsInUserGarden() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null) {
            User user = (User) authentication.getPrincipal();
            return gardenService.getUserPlants(user, null);
        }
        return new ArrayList<>();
    }



}
