package hdm.mi.growbros.service;

import hdm.mi.growbros.models.dto.CustomPageDto;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.search.PlantSpecification;
import hdm.mi.growbros.models.search.SearchRequest;
import hdm.mi.growbros.repositories.PlantRepository;
import lombok.RequiredArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@RequiredArgsConstructor
public class PlantsService {
    private final static Logger log = LoggerFactory.getLogger(PlantsService.class);
    private final static int MAX_PAGE_SIZE = 100;

    private final PlantRepository plantRepository;
    private final PlantSpecification plantSpecification;

    public CustomPageDto<Plant> getPlants(int pageNumber, int pageSize) {
        pageNumber = Math.max(0, pageNumber);
        pageSize = Math.min(MAX_PAGE_SIZE, pageSize);

        final Pageable page = PageRequest.of(pageNumber, pageSize);
        List<Plant> pageContent = plantRepository.findAll(page).getContent();
        return new CustomPageDto<>(pageNumber, pageContent.size(), pageContent);
    }

    public Collection<Plant> getRandomPlants(int numberOfPlants) {
        numberOfPlants= Math.min(MAX_PAGE_SIZE, numberOfPlants);
        long plantCount= plantRepository.count();
        int lastPage= (int)(plantCount/numberOfPlants)-1;
        int randomPage= (int) (Math.random()*lastPage);

        CustomPageDto<Plant> randomPlantPage= getPlants(randomPage, numberOfPlants);
        List<Plant> allPlantsOfPage = new ArrayList<>(randomPlantPage.content());
        Collections.shuffle(allPlantsOfPage);

        return allPlantsOfPage;
    }

    public Collection<Plant> getPlantsSearchedFor(SearchRequest searchRequest) {
        Specification<Plant> spec = plantSpecification.withSearchCriteria(searchRequest);
        return plantRepository.findAll(spec);
    }

    public void createPlant(Plant plant) {
        log.info("Saving plant '{}' to repository.", plant.getName());
        plantRepository.save(plant);
    }

    public Optional<Plant> findById(long id) {
        return plantRepository.findById(id);
    }
}
